# Generated by Django 3.0.6 on 2020-05-06 14:44

from django.db import migrations, models
import django.db.models.deletion
import django_geosource.models


class Migration(migrations.Migration):

    dependencies = [
        ("django_geosource", "0015_auto_20190927_1445"),
    ]

    operations = [
        migrations.CreateModel(
            name="CSVSource",
            fields=[
                (
                    "source_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="django_geosource.Source",
                    ),
                ),
                ("file", models.FileField(upload_to="geosource/csv/%Y")),
            ],
            options={"abstract": False, "base_manager_name": "objects"},
            bases=("django_geosource.source",),
        ),
        migrations.AlterField(
            model_name="source",
            name="geom_type",
            field=models.IntegerField(
                choices=[
                    (0, django_geosource.models.GeometryTypes["Point"]),
                    (1, django_geosource.models.GeometryTypes["LineString"]),
                    (3, django_geosource.models.GeometryTypes["Polygon"]),
                    (4, django_geosource.models.GeometryTypes["MultiPoint"]),
                    (5, django_geosource.models.GeometryTypes["MultiLineString"]),
                    (6, django_geosource.models.GeometryTypes["MultiPolygon"]),
                    (7, django_geosource.models.GeometryTypes["GeometryCollection"]),
                    (8, django_geosource.models.GeometryTypes["Undefined"]),
                ]
            ),
        ),
    ]
