# Generated by Django 2.0.13 on 2019-05-24 09:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("django_geosource", "0001_initial")]

    operations = [
        migrations.AlterField(
            model_name="postgissource",
            name="refresh",
            field=models.IntegerField(default=-1),
        )
    ]
