# Generated by Django 2.0.13 on 2019-06-28 12:49

import django_geosource.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("django_geosource", "0010_wmtssource")]

    operations = [
        migrations.AlterField(
            model_name="wmtssource",
            name="url",
            field=django_geosource.fields.LongURLField(),
        )
    ]
