# Generated by Django 2.0.13 on 2019-08-21 14:48

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("django_geosource", "0013_auto_20190709_1735")]

    operations = [
        migrations.AddField(
            model_name="source",
            name="created_at",
            field=models.DateTimeField(
                auto_now_add=True, default=django.utils.timezone.now
            ),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name="source",
            name="updated_at",
            field=models.DateTimeField(auto_now=True),
        ),
    ]
