# Generated by Django 2.1.12 on 2019-09-27 12:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("django_geosource", "0014_auto_20190821_1648")]

    operations = [
        migrations.RenameField(
            model_name="source", old_name="status", new_name="task_id"
        ),
        migrations.AddField(
            model_name="source", name="task_date", field=models.DateTimeField(null=True)
        ),
    ]
