# Generated by Django 2.0.13 on 2019-06-21 13:59

import django.contrib.postgres.fields.jsonb
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("django_geosource", "0007_commandsource")]

    operations = [
        migrations.AddField(
            model_name="source",
            name="settings",
            field=django.contrib.postgres.fields.jsonb.JSONField(default=dict),
        )
    ]
